*-----------------------------------------------------------
* Program    :Drawing Board
* Written by :Muhammad Ali Awan
* Date       :Sunday, May 04, 2008
* version    :1.0
* Description:After the initialization of the various aspects,
* like interrupts, generating the predefined colors, current 
* selected color box etc, application goes to an infinite loop, 
* that check for the keys, if there is any key pressed it jumps 
* to various branches for the proper handling of the key pressed, 
* meaning if 'r' key is pressed it increases the Red color factor 
* in the current selected color and redraw the new color. Similarly 
* if we press one of the key for the basic operations it goes to the
* routine that selects the basic operation and also you can see the 
* selected operation shape in upper left corner. Mouse is implemented 
* as in interrupt and an interrupt handler routine is written to service 
* the current interrupts. Mouse handler function first checks the type 
* of click (left, right or middle) and then calls the corresponding
* routine for the proper functions. Trap 15 is mostly used for all 
* the I/O.  Code is attached for the reference. 
*all Operations works with left click except irregular line as 
*irregular line works with middle mouse click.
*-----------------------------------------------------------
  ORG     $1000

START 
        move.l  #BUTTON_DOWN_IRQ,$64       ; IRQ1
        andi.w  #$00,SR                 ; put CPU in User mode 

        move.b  #60,d0          ; set mouse IRQ
        move.w  #$0101,d1       ; IRQ1 for mouse down
        trap    #15


*        move.b  #33,d0          ; set window size
*        move.l  #(800<<16+600),d1       ; 800 x 600
*        trap    #15
       
        move.b      #33,d0          ; set window size
        move.l      #(950<<16+700),d1      ; 1024 x 768
        trap        #15


        bsr heading             ; heading of the program
        bsr control_keys     ; controls keys
        bsr color_platte_1        ;default colors fill colors
        bsr color_platte_2
        bsr circle_custom

        
loop    nop

        bsr check_char 
        
        bsr char_handle        
        bra     loop    
        bra     exit

******************Subroutines*************************
* IRQ HANDLERS

* MOUSE BUTTON DOWN HANDLER
BUTTON_DOWN_IRQ
        movem.l d0-d1,-(a7)
        move.b  #2,d1           ; mouse down state
        move.b  #61,d0          ; read mouse
        trap    #15
        bsr     check_down_status
        movem.l (a7)+,d0-d1
        rte
        

check_middle
        lea operation,a5
        move.b (a5),d5
        lea flag_move,a6
        move.b  (a6),d6 
        
        if.b d5 <eq> #5
        
             if.b d6 <eq> #0 then.s        ; if flag=0 then store first point
                lea first_point,a1      ; store first point in first_point variable
                move.l d1,(a1)  
                move.b #1,(a6)          ;set flag_move=1
             endi
       
             if.b d6 <eq> #1 then.s        ;if flag=1 then do the desired operation       
                lea second_point,a1      ; store first point in first_point variable
                move.l d1,(a1)  
                bsr draw_line_move
                lea operation,a5
                move.b #5,(a5)
                lea first_point,a1
                lea second_point,a2
                move.l (a2),(a1)
             endi
        endi  
        rts
draw_line_move
           bsr set_pen_color
           bsr set_pen_width
           lea first_point,a3    ; x1=d1.w, y1=d2.w, 
           move.l (a3), d1
           move.l d1,d2
           swap d2
           lea second_point,a4   ; x2=d3.w, y2=d4.w, 
           move.l (a4),d3
           move.l d3,d4
           swap d4
                   
           move.l #84,d0   ; draw line b/w (x1,y1),(x2,y2)
           trap #15
           bsr set_default_pen_width
        rts
check_down_status

        move.b #2,d4    ;check for right mouse click
        and.b d0,d4
        cmp.b #2,d4
        bne left_click_test         
        swap d1
        move.w d1,d2
        swap d1 
        move #83,d0
        trap #15
        bsr set_new_colors
        bra out_down        
        
left_click_test move.b #1,d4    ;check for left mouse click
                and.b d0,d4
                cmp.b #1,d4
                bne middle_click_test
                bsr check_operation 
                bra out_down

middle_click_test move.b #4,d4    ;check for middle mouse click
                and.b d0,d4
                cmp.b #4,d4
                bne double_click_test
                bsr check_middle
                bra out_down

double_click_test nop 
                bra out_down        
        
out_down        nop
        rts

check_operation
        lea operation,a5
        move.b (a5),d5
        lea flag,a6
        move.b  (a6),d6 
        
        if.b d5 <ne> #0
        
             if.b d6 <eq> #0 then.s        ; if flag=0 then store first point
                lea first_point,a1      ; store first point in first_point variable
                move.l d1,(a1)  
                move.b #1,(a6)          ;set flag=1
             endi
       
             if.b d6 <eq> #1 then.s        ;if flag=1 then do the desired operation       
                lea second_point,a1      ; store first point in first_point variable
                move.l d1,(a1)  
                bsr draw_shape
                lea flag,a6
                move.b #0,(a6)          ;set flag=0
                lea operation,a5
                move.b #0,(a5)
                bsr clean
             endi
        endi     
        rts


draw_shape
        
                if.b d5 <eq> #0 then.s      ; if '0' pressed LINE
                       nop
                     
                endi
        
                if.b d5 <eq> #1 then.s      ; if '1' pressed Circle
                        bsr set_pen_color
                        bsr set_fill_color
                        lea first_point,a3    ; x1=d1.w, y2=d2.w, 
                        move.l (a3), d1
                        move.l d1,d2
                        swap d2
                        lea second_point,a4   ; x2=(x1+(y2-y1))=d3.w, y2=d4.w, 
                        move.l (a4),d3
                        move.l d3,d4
                        swap d4
                        clr.l d3        ;  x2=(x1+(y2-y1))=d3.w
                        move.w d4,d3 
                        sub.w d2,d3
                        add.w d1,d3
                        move.l #88,d0   ; draw circle b/w (x1,y1),(x2,y2)
                        trap #15                         
                endi
        
                if.b d5 <eq> #2 then.s      ; if '2' pressed square
                        bsr set_pen_color
                        bsr set_fill_color
                        lea first_point,a3    ; x1=d1.w, y2=d2.w, 
                        move.l (a3), d1
                        move.l d1,d2
                        swap d2
                        lea second_point,a4   ; x2=(x1+(y2-y1))=d3.w, y2=d4.w, 
                        move.l (a4),d3
                        move.l d3,d4
                        swap d4
                        clr.l d3        ;  x2=(x1+(y2-y1))=d3.w
                        move.w d4,d3 
                        sub.w d2,d3
                        add.w d1,d3
                        move.l #87,d0   ; draw square b/w (x1,y1),(x2,y2)
                        trap #15          
                endi
        
                if.b d5 <eq> #3 then.s      ; if '3' pressed RECTANGLE
                        bsr set_pen_color
                        bsr set_fill_color
                        lea first_point,a3    ; LX=d1.w, UY=d2.w, 
                        move.l (a3), d1
                        move.l d1,d2
                        swap d2
                        lea second_point,a4   ; RX=d3.w, LY=d4.w, 
                        move.l (a4),d3
                        move.l d3,d4
                        swap d4
                        
                        move.l #87,d0   ; draw rectangle b/w (x1,y1),(x2,y2)
                        trap #15
   
                endi
        
                if.b d5 <eq> #4 then.s      ; if '4' pressed ELLIPSE
                        bsr set_pen_color
                        bsr set_fill_color
                        lea first_point,a3    ; LX=d1.w, UY=d2.w, 
                        move.l (a3), d1
                        move.l d1,d2
                        swap d2
                        lea second_point,a4   ; RX=d3.w, LY=d4.w, 
                        move.l (a4),d3
                        move.l d3,d4
                        swap d4
                        
                        move.l #88,d0   ; draw ellipse b/w (x1,y1),(x2,y2)
                        trap #15 
                endi
        
*                if.b d5 <eq> #5 then.s      ; if '5' pressed IRREGULAR LINE

*                endi
        
                if.b d5 <eq> #6 then.s      ; if '6' pressed STRAIGHT LINE
                        bsr set_pen_color
                        bsr set_pen_width
                        lea first_point,a3    ; x1=d1.w, y1=d2.w, 
                        move.l (a3), d1
                        move.l d1,d2
                        swap d2
                        lea second_point,a4   ; x2=d3.w, y2=d4.w, 
                        move.l (a4),d3
                        move.l d3,d4
                        swap d4
                        
                        move.l #84,d0   ; draw line b/w (x1,y1),(x2,y2)
                        trap #15
                        bsr set_default_pen_width
                        
                endi

        rts
set_pen_width
        lea pen_width,a3
        move.b (a3),d1
        move.b #93,d0
        trap #15
       rts 
set_default_pen_width
        move.b #1,d1
        move.b #93,d0
        trap #15
       rts        
set_pen_color
        lea r,a4
        lea g,a5        
        lea b,a6
        clr.l d1
        move.b (a6),d1
        swap d1
        move.b (a5),d1
        lsl.w #8,d1
        move.b (a4),d1
        move.l #80,d0
        trap #15
       
       rts
       
set_fill_color
        lea r,a4
        lea g,a5        
        lea b,a6
        clr.l d1
        move.b (a6),d1
        swap d1
        move.b (a5),d1
        lsl.w #8,d1
        move.b (a4),d1
        move.l #81,d0
        trap #15
       
       rts       
                
set_new_colors

        lea r,a4
        lea g,a5        
        lea b,a6
        move.b d0,(a4)
        lsr.w #8,d0
        move.b d0,(a5)
        swap d0
        move.b d0,(a6)
        bsr circle_custom
      
       rts
 




color_platte_1
        bsr set_default_pen_color
        clr.l d2
        move.l #5,d2
        clr.l d4
        move.l #25,d4 
        
        clr.l d6
        move.l #150,d6
        clr.l d7
        move.l #170,d7
       
        
        move.b #16,d5   ;no of default colors
        clr.b d3
        lea COLORS,a1
repeat  move.l (a1)+,d1
        bsr trap_fill_color
        
        bsr select_rectangle_points
        bsr draw_rectangle1
        
        sub.b #1,d5
        cmp.b #0,d5
        bne repeat
      
      rts        
color_platte_2
        bsr set_default_pen_color
        clr.l d2
        move.l #35,d2
        clr.l d4
        move.l #55,d4 
        
        clr.l d6
        move.l #150,d6
        clr.l d7
        move.l #170,d7
       
        
        move.b #16,d5   ;no of default colors
        clr.b d3
        lea COLORS,a1
repeat1  move.l (a1)+,d1
        lsr.l #1,d1
        bsr trap_fill_color
        
        bsr select_rectangle_points
        bsr draw_rectangle1
        
        sub.b #1,d5
        cmp.b #0,d5
        bne repeat1
      
      rts   
select_rectangle_points
        move.l d6,d1     ;Lx
        add.l #30,d6
        move.l d7,d3     ;Rx
        add.l #30,d7
      
       rts 
draw_rectangle1
        move.l #87,d0     ;rectangle
        trap #15        
        
      rts         
set_default_pen_color
        move.l #$00f0f0f0,d1
        move.l #80,d0   ;set pen color
        trap #15

       rts

trap_fill_color
        move.l #81,d0   ;set fill color
        trap #15               
       
       rts
       
check_char
        move.b  #7,d0           ; check for pending key press
        trap    #15
       
       rts
        
circle_custom
        lea r,a1
        lea g,a2
        lea b,a3
        clr.l d5
        clr.l d6
        clr.l d7
        move.b (a1),d5
        move.b (a2),d6
        move.b (a3),d7
        lsl.l #8,d6
        swap d7
        clr.l d1
        or.l d5,d1
        or.l d6,d1
        or.l d7,d1
        move.l #81,d0   ;set fill color
        trap #15
        bsr set_default_pen_color

        
        move.l #100,d1     ;Lx
        move.l #5,d2     ;Uy
        move.l #120,d3     ;Rx
        move.l #25,d4     ;Ly
        move.l #88,d0     ;eclipse
        trap #15
        
       rts


 
add_red
          lea r,a1
          move.b (a1),d5
          addi.b #1,d5
          move.b d5,(a1)
            
        rts

add_blue
          lea b,a1
          move.b (a1),d6
          addi.b #1,d6
          move.b d6,(a1)
        rts 

add_green
          lea g,a1
          move.b (a1),d7
          addi.b #1,d7
          move.b d7,(a1)
        rts                       
            
read_and_disable_echo
          move.b #0,d1          ; disable echo of the key
          move.b        #12,d0
          trap  #15      
          move.b        #5,d0   ; read key
          trap  #15
          
        rts
        
char_handle
          if.b d1 <ne> #0 then.s  ; if key is ready
            bsr read_and_disable_echo      
            bsr select_shape
            bsr circle_custom
          endi
          nop        
        
        rts


blacky
        move.l #$00000000,d1
        move.l #80,d0   ;set pen color
        trap #15
        move.l #$00000000,d1
        move.l #81,d0   ;set fill color
        trap #15  
        rts
grey
        move.l #$00afafaf,d1
        move.l #80,d0   ;set pen color
        trap #15
        move.l #$00000000,d1
        move.l #81,d0   ;set fill color
        trap #15  
        rts        
clean
        bsr blacky
        move.l #5,d1     ;Lx
        move.l #15,d2     ;Uy
        move.l #35,d3     ;Rx
        move.l #45,d4     ;Ly
        move.l #87,d0     ;rectangle
        trap #15  
        
    rts

points 
        move.l #6,d1     ;Lx
        move.l #16,d2     ;Uy
        move.l #34,d3     ;Rx
        move.l #44,d4     ;Ly  
        rts   
points_rectangle_ellipse 
        move.l #6,d1     ;Lx
        move.l #22,d2     ;Uy
        move.l #34,d3     ;Rx
        move.l #38,d4     ;Ly  
        rts  
 
draw_line
        bsr grey
        bsr points
        move.l #84,d0    
        trap #15
        bsr blacky
    rts        

draw_square
        bsr grey
        bsr points
        move.l #87,d0   
        trap #15
        bsr blacky
        
    rts        
draw_circle
        bsr grey
        bsr points
        move.l #88,d0    
        trap #15
        bsr blacky
        
    rts  
draw_ellipse
        bsr grey
        bsr points_rectangle_ellispe
        move.l #88,d0    
        trap #15
        bsr blacky
    rts  
draw_rectangle
        bsr grey
        bsr points_rectangle_ellispe
        move.l #87,d0    
        trap #15
        bsr blacky
    rts
draw_irregular
        bsr grey
        
        move.l #6,d1     ;Lx
        move.l #16,d2     ;Uy
        move.l #8,d3     ;Rx
        move.l #24,d4     ;Ly  
        move.l #84,d0    
        trap #15
        move.l #8,d1     ;Lx
        move.l #24,d2     ;Uy
        move.l #13,d3     ;Rx
        move.l #26,d4     ;Ly  
        move.l #84,d0    
        trap #15  
        move.l #13,d1     ;Lx
        move.l #26,d2     ;Uy
        move.l #24,d3     ;Rx
        move.l #37,d4     ;Ly  
        move.l #84,d0    
        trap #15
        move.l #24,d1     ;Lx
        move.l #37,d2     ;Uy
        move.l #6,d3     ;Rx
        move.l #34,d4     ;Ly  
        move.l #84,d0    
        trap #15 
        move.l #6,d1     ;Lx
        move.l #34,d2     ;Uy
        move.l #34,d3     ;Rx
        move.l #44,d4     ;Ly  
        move.l #84,d0    
        trap #15           
        bsr blacky
    

add_width
        lea pen_width,a3
        move.b (a3),d7
        add.b #1,d7
        move.b d7,(a3)
       rts
sub_width               
        lea pen_width,a3
        move.b (a3),d7
        sub.b #1,d7
        move.b d7,(a3)
       rts
select_shape
          
        if.b d1 <eq> #'r' then.s      ; if 'r' pressed
            bsr add_red
            bsr clean
        endi
          
        if.b d1 <eq> #'b' then.s      ; if 'b' pressed
            bsr add_blue    
            bsr clean           
        endi
          
        if.b d1 <eq> #'g' then.s      ; if 'g' pressed
            bsr add_green    
            bsr clean            
        endi
        
        if.b d1 <eq> #'m' then.s      ; if 'm' pressed
            bsr add_width
            bsr clean
        endi

        if.b d1 <eq> #'n' then.s      ; if 'n' pressed
            bsr sub_width
            bsr clean
        endi
        
        if.b d1 <eq> #'0' then.s      ; if '0' pressed LINE
            lea operation, a3
            move.b #0,(a3) 
            lea flag_move,a6
            move.b #0,(a6)          ;set flag=0
            bsr clean
        endi
        
        if.b d1 <eq> #'1' then.s      ; if '1' pressed Circle
            lea operation, a3
            move.b #1,(a3)    
            bsr clean
            bsr draw_circle
        endi
        
        if.b d1 <eq> #'2' then.s      ; if '2' pressed square
            lea operation, a3
            move.b #2,(a3) 
            bsr clean
            bsr draw_square
        endi
        
        if.b d1 <eq> #'3' then.s      ; if '3' pressed RECTANGLE
            lea operation, a3
            move.b #3,(a3)    
            bsr clean
            bsr draw_rectangle
        endi
        
        if.b d1 <eq> #'4' then.s      ; if '4' pressed ELLIPSE
            lea operation, a3
            move.b #4,(a3) 
            bsr clean
            bsr draw_ellipse
        endi
        
        if.b d1 <eq> #'5' then.s      ; if '5' pressed IRREGULAR LINE
            lea operation, a3
            move.b #5,(a3)
            bsr draw_irregular
        endi
        
        if.b d1 <eq> #'6' then.s      ; if '6' pressed STRAIGHT LINE
            lea operation, a3
            move.b #6,(a3)
            bsr clean  
            bsr draw_line     
        endi
        
   
        rts

control_keys
        move.L  #$00FFFFFF,d1
        move.L  #$050c0003,d2
        move.B  #21,d0
        trap    #15
        
        move #0,d1
        move #680,d2
        move.b #11,d0
        trap #15
        
        lea key,a1
        move.b #95,d0
        trap #15        
      rts 



rts

heading
        
        move.L  #$000000FF,d1
        move.L  #$020A0007,d2
        move.B  #21,d0
        trap    #15
        
        lea     Red,a1
        MOVE.B  #14,D0
        TRAP    #15
        
        move.L  #$0000FF00,d1
        move.L  #$020A0007,d2
        move.B  #21,d0
        trap    #15
        
        lea     Green,a1
        MOVE.B  #14,D0
        TRAP    #15           
        
        
        move.L  #$00FF0000,d1
        move.L  #$020A0007,d2
        move.B  #21,d0
        trap    #15
        
        lea     Blue,a1
        MOVE.B  #14,D0
        TRAP    #15 
        
        rts     



**********Variables Declarations**************
Red     dc.b    'PA',0
Green   dc.b    'IN',0
Blue    dc.b    'TS',CR,LF,0

r       dc.b    230
g       dc.b    230
b       dc.b    230

operation    dc.b    0  ;select operation
flag         dc.b    0  ;first or second point 
flag_move    dc.b    0  ; point selection during move
first_point dc.l 0      ;first point
second_point dc.l 0     ;second point
pen_width    dc.b 10

carriage dc.b '',CR,0    
COLORS  DC.L    BLACK,MAROON,GREEN1,OLIVE,NAVY,PURPLE,TEAL,GRAY,RED1,LIME,YELLOW,BLUE1,FUCHSIA,AQUA,LTGRAY,WHITE
key dc.b             '(Cricle = 1)'
    dc.b             ' (Square = 2)'
    dc.b             ' (Rectangle = 3)'
    dc.b             ' (Ellipse = 4)'
    dc.b             ' (Ir Line = 5)'
    dc.b             ' (Line = 6)'
    dc.b             ' (Red = r)'
    dc.b             ' (Green = g)'
    dc.b             ' (Blue = b)'
    dc.b             ' (Inc = m)'
    dc.b             ' (Dec = n)',0
**********************************************


********** Constants Area*************        
CR      EQU     $0D
LF      EQU     $0A 
 
BLACK       EQU     $00000000
MAROON      EQU     $00000080
GREEN1      EQU     $00008000
OLIVE       EQU     $00008080
NAVY        EQU     $00800000
PURPLE      EQU     $00800080
TEAL        EQU     $00808000
GRAY        EQU     $00808080
RED1        EQU     $000000FF
LIME        EQU     $0000FF00
YELLOW      EQU     $0000FFFF
BLUE1       EQU     $00FF0000
FUCHSIA     EQU     $00FF00FF
AQUA        EQU     $00FFFF00
LTGRAY      EQU     $00C0C0C0
WHITE       EQU     $00FFFFFF

***************************************


exit    nop  
        MOVE.B  #9,D0
        TRAP    #15        
       
        END     START       





*~Font name~Courier New~
*~Font size~10~
*~Tab type~1~
*~Tab size~8~
